﻿local minToWarn = 50
local defaultThreshold = 3000
local defaultTracking = true

--[[                                      CurrencyMonitor

This program watches a player's justice points, valor points, conquest points, and honor points
and flashes a raid warning with optional raid warning sound when any point gain above the minimum of 
50 makes the points total exceed its player-defined threshold. 

               *** CurrencyMonitor is Copyright 2011 by Malavice of Mal'Ganis US ***               

Please do not distribute altered copies of this add-on because it can make maintaining it a 
nightmare. If you want to see a feature built into it, please add a comment on the official 
download pages on curse.com and wowinterface.com.

For now, I will take bug reports and feature requests directly in the comments sections of the 
official release channels. Until such a time that I believe I need a ticket tracking system (which
hopefully never happens), I will simply answer bug reports/feature requests in comments sections.

---------------
Version History
---------------

07/08/2012 - V.2.0       Created the first in-game GUI. Bugs likely. Added slash command to hide or
						 show the GUI. 
07/02/2012 - V.1.2.3	 Removed an unnecessary registered event. Matched default values to 
						 description. Flipped the change log. Moved variables around in the file to
						 make maintaining it just a bit easier.
02/24/2012 - V.1.2.2	 Fixed a problem with the currency warning not updating to reflect the
						 current amount of points when points are awarded. Minor .toc file fix.
12/15/2011 - V.1.2.1	 Changed some variables to local to avoid polluting the global environment.
12/09/2011 - V.1.2       Spam bug fix. Also warns of only the type of currency that triggered the 
					     warning. Considerable code rewrite for less redundancy.
12/05/2011 - V.1.1       Removed BG spamminess through smart points announcement.
12/02/2011 - V.1.0       Now firing on the correct event.					 
12/02/2011 - V.0.1(BETA) (Initial release)

-----
To-Do
-----

Nap. Damn I love naps. Future plans might include a configurable minToWarn variable, but 50 seems to
be just right so I probably won't mess with it.

--- Credits ---

Credit goes to Wowhead user eidyia for the idea and additional input that helped hone this add-on's
functionality. Specifically, thank eidyia for the version 1.1 removal of BG spam and the 1.2 removal
of buggy spam!

Seerah and Dridzt of wowinterface.com also deserve some major thanks for helping me iron out the 
wrinkles in saving my variables in a table between sessions. I was just about to give up on the idea
but it was nice to see their encouragements. Thanks you two!

--]]

----------------- Know what you're doing before you edit anything below this line. -----------------

-- Add slash command configuration menu
SLASH_CURRENCYMONITOR1, SLASH_CURRENCYMONITOR2 = "/cm", "/currencymonitor";

function SlashCmdList.CURRENCYMONITOR(msg, editbox)
	CM_ToggleGUI()
end

-- Table to store currency IDs for querying the server
local currencies = {
	"395", -- Justice points
	"396", -- Valor points
	"392", -- Honor points
	"390"  -- Conquest points
}

-- Table to store currency names for display to the player
local allCurrency = {"Justice Points","Valor Points","Honor Points","Conquest Points"}

-- Table to store currency thresholds over which the player should be notified
local currencyThresholds = {}

-- An empty table that gets used more often than a $5 whore
local currentAmount = {}

-- Update all thresholds with the edit box values when the Save button is pressed
function CM_UpdateAll(v,j,c,h,vt,jt,ct,ht)

	currencyThresholds = {v:GetText(), j:GetText(), c:GetText(), h:GetText()}	
	
	CM_db["valor"].threshold = currencyThresholds[1]
	CM_db["justice"].threshold = currencyThresholds[2]
	CM_db["conquest"].threshold = currencyThresholds[3]
	CM_db["honor"].threshold = currencyThresholds[4]

	CM_db["valor"].tracked = vt:GetChecked()
	CM_db["justice"].tracked = jt:GetChecked()
	CM_db["conquest"].tracked = ct:GetChecked()
	CM_db["honor"].tracked = ht:GetChecked()
	
	CM_ToggleGUI()
end

-- Toggle playing the Raid Warning sound or not depending on the button's GetChecked() return
function CM_updateRWSound(self)
	
	if self:GetChecked() then
		CM_warningSound = true
		PlaySoundFile("Sound\\Interface\\RaidWarning.wav")
		RaidNotice_AddMessage(RaidWarningFrame, "Raid Warning sound is now ENABLED!", ChatTypeInfo["RAID_WARNING"])
	else
		CM_warningSound = false
		RaidNotice_AddMessage(RaidWarningFrame, "Raid Warning sound is now DISABLED!", ChatTypeInfo["RAID_WARNING"])
	end
end

-- Loop through and get the current amount of the four currencies we care about
local function getAllCurrencies()

	for i = 1, 4 do
		local _, amountNow = GetCurrencyInfo(currencies[i])
		table.insert(currentAmount, amountNow)
	end
end

-- Adds the warning message to the raid warning frame 
local function printOutput(triggerCurrency)

	for i = 1, 4 do
		if allCurrency[i] == triggerCurrency then
			if CM_warningSound then
				PlaySoundFile("Sound\\Interface\\RaidWarning.wav")
			end
			RaidNotice_AddMessage(RaidWarningFrame, "You have " .. currentAmount[i] .. " " .. allCurrency[i] .. "!", ChatTypeInfo["RAID_WARNING"])
		end
	end
end

-- The functions to run when our registered events fire
function CurrencyMonitor_OnEvent(self, event, arg1)
	
	if event == "CHAT_MESSAGE_CURRENCY" and tonumber(string.match(arg1, '%d+', -4)) >= minToWarn then		
		getAllCurrencies()
		-- Compare threshold of tracked currencies with the amount of currency that triggered the message
		for i = 1, 4 do
			local triggerCurrency = string.match(arg1, allCurrency[i])
			if currentAmount[i] >= currencyThresholds[i] and trackerTable[i] then
				printOutput(triggerCurrency)
			end
		end
	-- Empty the working tables for next use
	currentAmount = nil
	currentAmount = {}	
	end
	
	if (event == "ADDON_LOADED" and arg1 == "CurrencyMonitor") then
		-- If there isn't a table already in the saved variables from last session, create one
		if CM_db == nil then
			CM_db = {}
			-- Assign default values to the new table
			CM_db["valor"] = {["threshold"] = defaultThreshold, ["tracked"] = defaultTracking}
			CM_db["justice"] = {["threshold"] = defaultThreshold, ["tracked"] = defaultTracking}
			CM_db["conquest"] = {["threshold"] = defaultThreshold, ["tracked"] = defaultTracking}
			CM_db["honor"] = {["threshold"] = defaultThreshold, ["tracked"] = defaultTracking}
		end
		
		-- Define trackerTable for the CHAT_MSG_CURRENCY function to use
		local trackerTable = {CM_db["valor"].tracked,CM_db["justice"].tracked,CM_db["conquest"].tracked,CM_db["honor"].tracked}
		
		-- Load GUI with check boxes marked according to settings
		CM_ValorTrack:SetChecked(CM_db["valor"].tracked)
		CM_JusticeTrack:SetChecked(CM_db["justice"].tracked)
		CM_ConquestTrack:SetChecked(CM_db["conquest"].tracked)
		CM_HonorTrack:SetChecked(CM_db["honor"].tracked)
		
		-- Set the edit boxes' default values
		CM_ValorEdit:SetText(CM_db["valor"].threshold)
		CM_JusticeEdit:SetText(CM_db["justice"].threshold)
		CM_ConquestEdit:SetText(CM_db["conquest"].threshold)
		CM_HonorEdit:SetText(CM_db["honor"].threshold)
		
		-- Load the Raid Warning sound check box according to settings
		RWSoundSelect:SetChecked(CM_warningSound)
	end
end

-- Hide the GUI if it's shown, or show it if it's hidden
function CM_ToggleGUI(self)
	-- If the GUI is shown, hide it. If it's hidden, show it.
	if CurrencyMonitorFrame:IsVisible(self) then
		CurrencyMonitorFrame:Hide(self)
	else
		CurrencyMonitorFrame:Show(self)
	end
end